@@ -70,6 +70,10 @@ static gboolean xfce_backdrop_timer(XfceBackdrop *backdrop);
 
 static GdkPixbuf *xfce_backdrop_generate_canvas(XfceBackdrop *backdrop);
 
+static void xfce_backdrop_create_final_image(XfceBackdrop *backdrop);
+
+static gboolean xfce_backdrop_update_frame(XfceBackdrop *backdrop);
+
 static void xfce_backdrop_loader_size_prepared_cb(GdkPixbufLoader *loader,
                                                   gint width,
                                                   gint height,
@@ -103,12 +107,16 @@ struct _XfceBackdropPriv
     gint width, height;
     gint bpp;
 
+    GdkPixbufAnimation *animation;
+    GdkPixbufAnimationIter *animation_iter;
+    gint animation_timer;
+
     GdkPixbuf *pix;
     XfceBackdropImageData *image_data;
 
     XfceBackdropColorStyle color_style;
-    GdkColor color1;
-    GdkColor color2;
+    GdkRGBA color1;
+    GdkRGBA color2;
 
     XfceBackdropImageStyle image_style;
     gchar *image_path;
@@ -122,6 +130,7 @@ struct _XfceBackdropPriv
     guint cycle_timer_id;
     XfceBackdropCyclePeriod cycle_period;
     gboolean random_backdrop_order;
+    gboolean do_animations;
 };
 
 struct _XfceBackdropImageData
@@ -156,6 +165,7 @@ enum
     PROP_BACKDROP_CYCLE_PERIOD,
     PROP_BACKDROP_CYCLE_TIMER,
     PROP_BACKDROP_RANDOM_ORDER,
+    PROP_BACKDROP_DO_ANIMATIONS,
 };
 
 static guint backdrop_signals[LAST_SIGNAL] = { 0, };
@@ -163,88 +173,81 @@ static guint backdrop_signals[LAST_SIGNAL] = { 0, };
 /* helper functions */
 
 static GdkPixbuf *
-create_solid(GdkColor *color,
+create_solid(GdkRGBA *color,
              gint width,
-             gint height,
-             gboolean has_alpha,
-             gint alpha)
+             gint height)
 {
+    GdkWindow *root;
     GdkPixbuf *pix;
-    guint32 rgba;
-    
-    pix = gdk_pixbuf_new(GDK_COLORSPACE_RGB, has_alpha, 8, width, height);
-    
-    rgba = ((((color->red & 0xff00) << 8) | ((color->green & 0xff00))
-            | ((color->blue & 0xff00) >> 8)) << 8) | alpha;
-    
-    gdk_pixbuf_fill(pix, rgba);
+    cairo_surface_t *surface;
+    cairo_t *cr;
+
+    root = gdk_screen_get_root_window(gdk_screen_get_default ());
+    surface = gdk_window_create_similar_surface(root, CAIRO_CONTENT_COLOR_ALPHA, width, height);
+    cr = cairo_create(surface);
+
+    cairo_set_source_rgba(cr, color->red, color->green, color->blue, color->alpha);
+
+    cairo_rectangle(cr, 0, 0, width, height);
+    cairo_fill(cr);
+
+    cairo_surface_flush(surface);
+
+    pix = gdk_pixbuf_get_from_surface(surface, 0, 0, width, height);
+
+    cairo_destroy(cr);
+    cairo_surface_destroy(surface);
     
     return pix;
 }
 
 static GdkPixbuf *
-create_gradient(GdkColor *color1, GdkColor *color2, gint width, gint height,
+create_gradient(GdkRGBA *color1, GdkRGBA *color2, gint width, gint height,
         XfceBackdropColorStyle style)
 {
+    GdkWindow *root;
     GdkPixbuf *pix;
-    gint i, j;
-    GdkPixdata pixdata;
-    guint8 rgb[3];
-    GError *err = NULL;
-    
+    cairo_surface_t *surface;
+    cairo_pattern_t *pat;
+    cairo_t *cr;
+
     g_return_val_if_fail(color1 != NULL && color2 != NULL, NULL);
     g_return_val_if_fail(width > 0 && height > 0, NULL);
-    g_return_val_if_fail(style == XFCE_BACKDROP_COLOR_HORIZ_GRADIENT
-            || style == XFCE_BACKDROP_COLOR_VERT_GRADIENT, NULL);
-    
-    pixdata.magic = GDK_PIXBUF_MAGIC_NUMBER;
-    pixdata.length = GDK_PIXDATA_HEADER_LENGTH + (width * height * 3);
-    pixdata.pixdata_type = GDK_PIXDATA_COLOR_TYPE_RGB
-            | GDK_PIXDATA_SAMPLE_WIDTH_8 | GDK_PIXDATA_ENCODING_RAW;
-    pixdata.rowstride = width * 3;
-    pixdata.width = width;
-    pixdata.height = height;
-    pixdata.pixel_data = g_malloc(width * height * 3);
+    g_return_val_if_fail(style == XFCE_BACKDROP_COLOR_HORIZ_GRADIENT || style == XFCE_BACKDROP_COLOR_VERT_GRADIENT, NULL);
 
-    if(style == XFCE_BACKDROP_COLOR_HORIZ_GRADIENT) {
-        for(i = 0; i < width; i++) {
-            rgb[0] = (color1->red + (i * (color2->red - color1->red) / width)) >> 8;
-            rgb[1] = (color1->green + (i * (color2->green - color1->green) / width)) >> 8;
-            rgb[2] = (color1->blue + (i * (color2->blue - color1->blue) / width)) >> 8;
-            memcpy(pixdata.pixel_data+(i*3), rgb, 3);
-        }
-        
-        for(i = 1; i < height; i++) {
-            memcpy(pixdata.pixel_data+(i*pixdata.rowstride),
-                    pixdata.pixel_data, pixdata.rowstride);
-        }
-    } else if(XFCE_BACKDROP_COLOR_TRANSPARENT == style)
-        memset(pixdata.pixel_data, 0x0, width * height * 3);
-    else {
-        for(i = 0; i < height; i++) {
-            rgb[0] = (color1->red + (i * (color2->red - color1->red) / height)) >> 8;
-            rgb[1] = (color1->green + (i * (color2->green - color1->green) / height)) >> 8;
-            rgb[2] = (color1->blue + (i * (color2->blue - color1->blue) / height)) >> 8;
-            for(j = 0; j < width; j++)
-                memcpy(pixdata.pixel_data+(i*pixdata.rowstride)+(j*3), rgb, 3);
-        }
+    root = gdk_screen_get_root_window(gdk_screen_get_default ());
+    surface = gdk_window_create_similar_surface(root, CAIRO_CONTENT_COLOR_ALPHA, width, height);
+    cr = cairo_create(surface);
+
+    if(style == XFCE_BACKDROP_COLOR_VERT_GRADIENT) {
+        pat = cairo_pattern_create_linear (0.0, 0.0,  0.0, height);
+    } else {
+        pat = cairo_pattern_create_linear (0.0, 0.0,  width, 0.0);
     }
-    
-    pix = gdk_pixbuf_from_pixdata(&pixdata, TRUE, &err);
-    if(!pix) {
-        g_warning("%s: Unable to create color gradient: %s\n", PACKAGE,
-                err->message);
-        g_error_free(err);
-    }
-    
-    g_free(pixdata.pixel_data);
-    
+
+    cairo_pattern_add_color_stop_rgba (pat, 1, color2->red, color2->green, color2->blue, color2->alpha);
+    cairo_pattern_add_color_stop_rgba (pat, 0, color1->red, color1->green, color1->blue, color1->alpha);
+
+    cairo_rectangle(cr, 0, 0, width, height);
+    cairo_set_source(cr, pat);
+    cairo_fill(cr);
+
+    cairo_surface_flush(surface);
+
+    pix = gdk_pixbuf_get_from_surface(surface, 0, 0, width, height);
+
+    cairo_destroy(cr);
+    cairo_pattern_destroy(pat);
+    cairo_surface_destroy(surface);
+
     return pix;
 }
 
 void
 xfce_backdrop_clear_cached_image(XfceBackdrop *backdrop)
 {
+    TRACE("entering");
+
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
 
     if(backdrop->priv->pix == NULL)
@@ -254,6 +257,28 @@ xfce_backdrop_clear_cached_image(XfceBackdrop *backdrop)
     backdrop->priv->pix = NULL;
 }
 
+static void
+xfce_backdrop_clear_animation(XfceBackdrop *backdrop)
+{
+    TRACE("entering");
+
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    if(G_IS_OBJECT(backdrop->priv->animation)) {
+        g_object_unref(backdrop->priv->animation);
+        backdrop->priv->animation = NULL;
+    }
+
+    if(G_IS_OBJECT(backdrop->priv->animation_iter)) {
+        backdrop->priv->animation_iter = NULL;
+    }
+
+    if(backdrop->priv->animation_timer != 0) {
+        g_source_remove(backdrop->priv->animation_timer);
+        backdrop->priv->animation_timer = 0;
+    }
+}
+
 static void
 xfdesktop_backdrop_clear_directory_monitor(XfceBackdrop *backdrop)
 {
@@ -368,11 +393,6 @@ cb_xfce_backdrop_image_files_changed(GFileMonitor     *monitor,
                 backdrop->priv->image_files = g_list_delete_link(backdrop->priv->image_files, item);
 
             g_free(changed_file);
-
-            if (backdrop->priv->cycle_timer_id) {
-                g_source_remove(backdrop->priv->cycle_timer_id);
-                backdrop->priv->cycle_timer_id = 0;
-            }
             break;
         case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
             changed_file = g_file_get_path(file);
@@ -384,6 +404,7 @@ cb_xfce_backdrop_image_files_changed(GFileMonitor     *monitor,
                 DBG("match");
                 /* clear the outdated backdrop */
                 xfce_backdrop_clear_cached_image(backdrop);
+                xfce_backdrop_clear_animation(backdrop);
 
                 /* backdrop changed! */
                 g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
@@ -709,14 +730,14 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
                                     g_param_spec_boxed("first-color",
                                                        "first color",
                                                        "first color",
-                                                       GDK_TYPE_COLOR,
+                                                       GDK_TYPE_RGBA,
                                                        XFDESKTOP_PARAM_FLAGS));
 
     g_object_class_install_property(gobject_class, PROP_COLOR2,
                                     g_param_spec_boxed("second-color",
                                                        "second color",
                                                        "second color",
-                                                       GDK_TYPE_COLOR,
+                                                       GDK_TYPE_RGBA,
                                                        XFDESKTOP_PARAM_FLAGS));
 
     /* Defaults to an invalid image style so that
@@ -767,6 +788,13 @@ xfce_backdrop_class_init(XfceBackdropClass *klass)
                                                          FALSE,
                                                          XFDESKTOP_PARAM_FLAGS));
 
+    g_object_class_install_property(gobject_class, PROP_BACKDROP_DO_ANIMATIONS,
+                                    g_param_spec_boolean("backdrop-do-animations",
+                                                         "backdrop-do-animations",
+                                                         "backdrop-do-animations",
+                                                         FALSE,
+                                                         XFDESKTOP_PARAM_FLAGS));
+
 #undef XFDESKTOP_PARAM_FLAGS
 }
 
@@ -778,12 +806,14 @@ xfce_backdrop_init(XfceBackdrop *backdrop)
     backdrop->priv->cycle_timer_id = 0;
 
     /* color defaults */
-    backdrop->priv->color1.red = 0x1515;
-    backdrop->priv->color1.green = 0x2222;
-    backdrop->priv->color1.blue = 0x3333;
-    backdrop->priv->color2.red = 0x1515;
-    backdrop->priv->color2.green = 0x2222;
-    backdrop->priv->color2.blue = 0x3333;
+    backdrop->priv->color1.red = 0.08235f;
+    backdrop->priv->color1.green = 0.13333f;
+    backdrop->priv->color1.blue = 0.2f;
+    backdrop->priv->color1.alpha = 1.0f;
+    backdrop->priv->color2.red = 0.08235f;
+    backdrop->priv->color2.green = 0.13333f;
+    backdrop->priv->color2.blue = 0.2f;
+    backdrop->priv->color2.alpha = 1.0f;
 }
 
 static void
@@ -802,6 +832,7 @@ xfce_backdrop_finalize(GObject *object)
     }
 
     xfce_backdrop_clear_cached_image(backdrop);
+    xfce_backdrop_clear_animation(backdrop);
 
     xfdesktop_backdrop_clear_directory_monitor(backdrop);
 
@@ -821,7 +852,7 @@ xfce_backdrop_set_property(GObject *object,
                            GParamSpec *pspec)
 {
     XfceBackdrop *backdrop = XFCE_BACKDROP(object);
-    GdkColor *color;
+    GdkRGBA *color;
 
     switch(property_id) {
         case PROP_COLOR_STYLE:
@@ -865,6 +896,10 @@ xfce_backdrop_set_property(GObject *object,
             xfce_backdrop_set_random_order(backdrop, g_value_get_boolean(value));
             break;
 
+        case PROP_BACKDROP_DO_ANIMATIONS:
+            xfce_backdrop_set_do_animations(backdrop, g_value_get_boolean(value));
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
             break;
@@ -917,6 +952,10 @@ xfce_backdrop_get_property(GObject *object,
             g_value_set_boolean(value, xfce_backdrop_get_random_order(backdrop));
             break;
 
+        case PROP_BACKDROP_DO_ANIMATIONS:
+            g_value_set_boolean(value, xfce_backdrop_get_do_animations(backdrop));
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
             break;
@@ -1010,6 +1049,8 @@ void
 xfce_backdrop_set_color_style(XfceBackdrop *backdrop,
         XfceBackdropColorStyle style)
 {
+    TRACE("entering");
+
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
     g_return_if_fail((int)style >= -1 && style <= XFCE_BACKDROP_COLOR_TRANSPARENT);
 
@@ -1031,7 +1072,7 @@ xfce_backdrop_get_color_style(XfceBackdrop *backdrop)
 /**
  * xfce_backdrop_set_first_color:
  * @backdrop: An #XfceBackdrop.
- * @color: A #GdkColor.
+ * @color: A #GdkRGBA.
  *
  * Sets the "first" color for the #XfceBackdrop.  This is the color used if
  * the color style is set to XFCE_BACKDROP_COLOR_SOLID.  It is used as the
@@ -1041,35 +1082,38 @@ xfce_backdrop_get_color_style(XfceBackdrop *backdrop)
  **/
 void
 xfce_backdrop_set_first_color(XfceBackdrop *backdrop,
-                              const GdkColor *color)
+                              const GdkRGBA *color)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop) && color != NULL);
     
     if(color->red != backdrop->priv->color1.red
             || color->green != backdrop->priv->color1.green
-            || color->blue != backdrop->priv->color1.blue)
+            || color->blue != backdrop->priv->color1.blue
+            || color->alpha != backdrop->priv->color1.alpha)
     {
         xfce_backdrop_clear_cached_image(backdrop);
         backdrop->priv->color1.red = color->red;
         backdrop->priv->color1.green = color->green;
         backdrop->priv->color1.blue = color->blue;
+        backdrop->priv->color1.alpha = color->alpha;
+
         g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
     }
 }
 
 void
 xfce_backdrop_get_first_color(XfceBackdrop *backdrop,
-                              GdkColor *color)
+                              GdkRGBA *color)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop) && color);
     
-    memcpy(color, &backdrop->priv->color1, sizeof(GdkColor));
+    memcpy(color, &backdrop->priv->color1, sizeof(GdkRGBA));
 }
 
 /**
  * xfce_backdrop_set_second_color:
  * @backdrop: An #XfceBackdrop.
- * @color: A #GdkColor.
+ * @color: A #GdkRGBA.
  *
  * Sets the "second" color for the #XfceBackdrop.  This is the color used as the
  * right-side color or bottom color if the color style is set to
@@ -1078,18 +1122,21 @@ xfce_backdrop_get_first_color(XfceBackdrop *backdrop,
  **/
 void
 xfce_backdrop_set_second_color(XfceBackdrop *backdrop,
-                               const GdkColor *color)
+                               const GdkRGBA *color)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop) && color != NULL);
     
     if(color->red != backdrop->priv->color2.red
             || color->green != backdrop->priv->color2.green
-            || color->blue != backdrop->priv->color2.blue)
+            || color->blue != backdrop->priv->color2.blue
+            || color->alpha != backdrop->priv->color2.alpha)
     {
         xfce_backdrop_clear_cached_image(backdrop);
         backdrop->priv->color2.red = color->red;
         backdrop->priv->color2.green = color->green;
         backdrop->priv->color2.blue = color->blue;
+        backdrop->priv->color2.alpha = color->alpha;
+
         if(backdrop->priv->color_style != XFCE_BACKDROP_COLOR_SOLID)
             g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
     }
@@ -1097,11 +1144,11 @@ xfce_backdrop_set_second_color(XfceBackdrop *backdrop,
 
 void
 xfce_backdrop_get_second_color(XfceBackdrop *backdrop,
-                               GdkColor *color)
+                               GdkRGBA *color)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop) && color);
     
-    memcpy(color, &backdrop->priv->color2, sizeof(GdkColor));
+    memcpy(color, &backdrop->priv->color2, sizeof(GdkRGBA));
 }
 
 /**
@@ -1119,9 +1166,12 @@ xfce_backdrop_set_image_style(XfceBackdrop *backdrop,
                               XfceBackdropImageStyle style)
 {
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
-    
+
+    TRACE("entering");
+
     if(style != backdrop->priv->image_style) {
         xfce_backdrop_clear_cached_image(backdrop);
+        xfce_backdrop_clear_animation(backdrop);
         backdrop->priv->image_style = style;
         g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
     }
@@ -1188,6 +1238,7 @@ xfce_backdrop_set_image_filename(XfceBackdrop *backdrop, const gchar *filename)
         backdrop->priv->image_path = NULL;
 
     xfce_backdrop_clear_cached_image(backdrop);
+    xfce_backdrop_clear_animation(backdrop);
 
     xfce_backdrop_load_image_files(backdrop);
 
@@ -1526,6 +1577,50 @@ xfce_backdrop_get_random_order(XfceBackdrop *backdrop)
     return backdrop->priv->random_backdrop_order;
 }
 
+static gboolean
+xfce_backdrop_is_image_animated(XfceBackdrop *backdrop)
+{
+    if(backdrop->priv->animation != NULL &&
+       gdk_pixbuf_animation_is_static_image (backdrop->priv->animation) == FALSE &&
+       backdrop->priv->do_animations)
+    {
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
+/**
+ * xfce_backdrop_set_do_animations:
+ * @backdrop: An #XfceBackdrop.
+ * @animate: When TRUE and the backdrops will animate.
+ *
+ **/
+void
+xfce_backdrop_set_do_animations(XfceBackdrop *backdrop,
+                                gboolean do_animations)
+{
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    TRACE("entering");
+
+    if(backdrop->priv->do_animations != do_animations) {
+        backdrop->priv->do_animations = do_animations;
+
+        if(backdrop->priv->animation) {
+            xfce_backdrop_update_frame(backdrop);
+        }
+    }
+}
+
+gboolean
+xfce_backdrop_get_do_animations(XfceBackdrop *backdrop)
+{
+    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), FALSE);
+
+    return backdrop->priv->do_animations;
+}
+
 void
 xfce_backdrop_force_cycle(XfceBackdrop *backdrop)
 {
@@ -1548,9 +1643,13 @@ xfce_backdrop_generate_canvas(XfceBackdrop *backdrop)
     gint w, h;
     GdkPixbuf *final_image;
 
+    TRACE("entering");
+
     w = backdrop->priv->width;
     h = backdrop->priv->height;
 
+    DBG("w %d h %d", w, h);
+
     /* In case we somehow end up here, give a warning and apply a temp fix */
     if(backdrop->priv->color_style == XFCE_BACKDROP_COLOR_INVALID) {
         g_warning("xfce_backdrop_generate_canvas: Invalid color style");
@@ -1558,15 +1657,15 @@ xfce_backdrop_generate_canvas(XfceBackdrop *backdrop)
     }
 
     if(backdrop->priv->color_style == XFCE_BACKDROP_COLOR_SOLID)
-        final_image = create_solid(&backdrop->priv->color1, w, h, FALSE, 0xff);
+        final_image = create_solid(&backdrop->priv->color1, w, h);
     else if(backdrop->priv->color_style == XFCE_BACKDROP_COLOR_TRANSPARENT) {
-        GdkColor c = { 0, 0xffff, 0xffff, 0xffff };
-        final_image = create_solid(&c, w, h, TRUE, 0x00);
+        GdkRGBA c = { 1.0f, 1.0f, 1.0f, 1.0f };
+        final_image = create_solid(&c, w, h);
     } else {
         final_image = create_gradient(&backdrop->priv->color1,
                 &backdrop->priv->color2, w, h, backdrop->priv->color_style);
         if(!final_image)
-            final_image = create_solid(&backdrop->priv->color1, w, h, FALSE, 0xff);
+            final_image = create_solid(&backdrop->priv->color1, w, h);
     }
 
     return final_image;
@@ -1580,9 +1679,12 @@ xfce_backdrop_image_data_release(XfceBackdropImageData *image_data)
     if(!image_data)
         return;
 
-    /* Only set the backdrop's image_data to NULL if it's current */
-    if(image_data->backdrop->priv->image_data == image_data)
-        image_data->backdrop->priv->image_data = NULL;
+    if(XFCE_IS_BACKDROP(image_data->backdrop)) {
+        /* Only set the backdrop's image_data to NULL if it's current */
+        if(image_data->backdrop->priv->image_data == image_data) {
+            image_data->backdrop->priv->image_data = NULL;
+        }
+    }
 
     if(image_data->cancellable)
         g_object_unref(image_data->cancellable);
@@ -1612,6 +1714,11 @@ xfce_backdrop_get_pixbuf(XfceBackdrop *backdrop)
         return g_object_ref(backdrop->priv->pix);
     }
 
+    if(xfce_backdrop_is_image_animated(backdrop)) {
+        xfce_backdrop_create_final_image(backdrop);
+        return g_object_ref(backdrop->priv->pix);
+    }
+
     /* !backdrop->priv->pix, call xfce_backdrop_generate_async */
     return NULL;
 }
@@ -1642,10 +1749,10 @@ xfce_backdrop_generate_async(XfceBackdrop *backdrop)
         backdrop->priv->image_data = NULL;
     }
 
-    /* If we aren't going to display an image then just create the canvas */
+    /* If we aren't going to display an image then just skip to creating the canvas */
     if(backdrop->priv->image_style == XFCE_BACKDROP_IMAGE_NONE) {
-        backdrop->priv->pix = xfce_backdrop_generate_canvas(backdrop);
-        g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_READY], 0);
+        XF_DEBUG("image style == NONE");
+        xfce_backdrop_create_final_image(backdrop);
         return;
     }
 
@@ -1686,11 +1793,30 @@ xfce_backdrop_loader_size_prepared_cb(GdkPixbufLoader *loader,
                                       gpointer user_data)
 {
     XfceBackdropImageData *image_data = user_data;
-    XfceBackdrop *backdrop = image_data->backdrop;
+    XfceBackdrop *backdrop;
     gdouble xscale, yscale;
 
     TRACE("entering");
 
+    if(image_data == NULL)
+        return;
+
+    backdrop = image_data->backdrop;
+
+    /* canceled? quit now */
+    if(g_cancellable_is_cancelled(image_data->cancellable)) {
+        xfce_backdrop_image_data_release(image_data);
+        g_free(image_data);
+        return;
+    }
+
+    /* invalid backdrop? quit */
+    if(!XFCE_IS_BACKDROP(backdrop)) {
+        xfce_backdrop_image_data_release(image_data);
+        g_free(image_data);
+        return;
+    }
+
     if(backdrop->priv->image_style == XFCE_BACKDROP_IMAGE_INVALID) {
         g_warning("Invalid image style, setting to XFCE_BACKDROP_IMAGE_ZOOMED");
         backdrop->priv->image_style = XFCE_BACKDROP_IMAGE_ZOOMED;
@@ -1699,6 +1825,7 @@ xfce_backdrop_loader_size_prepared_cb(GdkPixbufLoader *loader,
     switch(backdrop->priv->image_style) {
         case XFCE_BACKDROP_IMAGE_CENTERED:
         case XFCE_BACKDROP_IMAGE_TILED:
+        case XFCE_BACKDROP_IMAGE_NONE:
             /* do nothing */
             break;
 
@@ -1742,12 +1869,31 @@ xfce_backdrop_loader_size_prepared_cb(GdkPixbufLoader *loader,
     }
 }
 
-static void
-xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
-                               XfceBackdropImageData *image_data)
+static gboolean
+xfce_backdrop_update_frame(XfceBackdrop *backdrop)
 {
-    XfceBackdrop *backdrop = image_data->backdrop;
-    GdkPixbuf *final_image, *image, *tmp;
+    g_return_val_if_fail(XFCE_IS_BACKDROP(backdrop), G_SOURCE_REMOVE);
+
+    /* kill the old timer, we won't bother with frame updates until
+     * a frame is loaded to cut down on CPU usage */
+    if(backdrop->priv->animation_timer != 0) {
+            g_source_remove(backdrop->priv->animation_timer);
+            backdrop->priv->animation_timer = 0;
+    }
+
+    /* old image is invalid now, we'll generate a new one when needed */
+    xfce_backdrop_clear_cached_image(backdrop);
+
+    /* backdrop changed! */
+    g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_CHANGED], 0);
+
+    return G_SOURCE_REMOVE;
+}
+
+static void
+xfce_backdrop_create_final_image(XfceBackdrop *backdrop)
+{
+    GdkPixbuf *final_image = NULL, *image = NULL, *tmp = NULL;
     gint i, j;
     gint w, h, iw = 0, ih = 0;
     XfceBackdropImageStyle istyle;
@@ -1759,14 +1905,20 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
 
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
 
-    /* canceled? quit now */
-    if(g_cancellable_is_cancelled(image_data->cancellable)) {
-        xfce_backdrop_image_data_release(image_data);
-        g_free(image_data);
-        return;
+    xfce_backdrop_clear_cached_image(backdrop);
+
+    if(backdrop->priv->animation != NULL) {
+        if(!xfce_backdrop_is_image_animated(backdrop)) {
+            /* Get a reasonable thing to display as a static unanimated image */
+            DBG("static image");
+            image = gdk_pixbuf_animation_get_static_image(backdrop->priv->animation);
+        } else {
+            DBG("animation");
+            gdk_pixbuf_animation_iter_advance(backdrop->priv->animation_iter, NULL);
+            image = gdk_pixbuf_animation_iter_get_pixbuf(backdrop->priv->animation_iter);
+        }
     }
 
-    image = gdk_pixbuf_loader_get_pixbuf(loader);
     if(image) {
         /* If the image is supposed to be rotated, do that now */
         GdkPixbuf *temp = gdk_pixbuf_apply_embedded_orientation (image);
@@ -1810,20 +1962,24 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
 
     final_image = xfce_backdrop_generate_canvas(backdrop);
 
-    /* no image and not canceled? return just the canvas */
-    if(!image && !g_cancellable_is_cancelled(image_data->cancellable)) {
-        XF_DEBUG("image failed to load, displaying canvas only");
+    /* no image? return just the canvas */
+    if(!image) {
+        if(istyle != XFCE_BACKDROP_IMAGE_NONE) {
+            XF_DEBUG("image failed to load, displaying canvas only");
+        } else {
+            XF_DEBUG("style == NONE, only displaying canvas");
+        }
         backdrop->priv->pix = final_image;
 
         g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_READY], 0);
-
-        backdrop->priv->image_data = NULL;
-        xfce_backdrop_image_data_release(image_data);
-        g_free(image_data);
         return;
     }
 
     switch(istyle) {
+        case XFCE_BACKDROP_IMAGE_NONE:
+            /* do nothing */
+            break;
+
         case XFCE_BACKDROP_IMAGE_CENTERED:
             dx = MAX((w - iw) / 2, 0);
             dy = MAX((h - ih) / 2, 0);
@@ -1909,17 +2065,60 @@ xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
             g_critical("Invalid image style: %d\n", (gint)istyle);
     }
 
-    /* keep the backdrop and emit the signal if it hasn't been canceled */
-    if(!g_cancellable_is_cancelled(image_data->cancellable)) {
-        backdrop->priv->pix = final_image;
+    /* keep the backdrop and emit the signal */
+    backdrop->priv->pix = final_image;
+    if(!xfce_backdrop_is_image_animated(backdrop)) {
+        /* We're only sending the ready signal for static images, animations
+         * are updated with frame updates */
         g_signal_emit(G_OBJECT(backdrop), backdrop_signals[BACKDROP_READY], 0);
     }
 
+
     /* We either created image or took a ref with
      * gdk_pixbuf_apply_embedded_orientation, free it here
      */
     if(image)
         g_object_unref(image);
+
+    /* do we need to set a new timer for the next frame? */
+    if(xfce_backdrop_is_image_animated(backdrop)) {
+        gint timeout;
+
+        timeout = gdk_pixbuf_animation_iter_get_delay_time(backdrop->priv->animation_iter);
+        DBG("animation timeout %d", timeout);
+
+        if(timeout > 0) {
+            backdrop->priv->animation_timer = g_timeout_add(timeout,
+                                                            (GSourceFunc)xfce_backdrop_update_frame,
+                                                            backdrop);
+        }
+    }
+}
+
+static void
+xfce_backdrop_loader_closed_cb(GdkPixbufLoader *loader,
+                               XfceBackdropImageData *image_data)
+{
+    XfceBackdrop *backdrop = image_data->backdrop;
+
+    TRACE("entering");
+
+    g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
+
+    /* canceled? quit now */
+    if(g_cancellable_is_cancelled(image_data->cancellable)) {
+        xfce_backdrop_image_data_release(image_data);
+        g_free(image_data);
+        return;
+    }
+
+    backdrop->priv->animation = gdk_pixbuf_loader_get_animation(loader);
+    if(GDK_IS_PIXBUF_ANIMATION(backdrop->priv->animation))
+        backdrop->priv->animation_iter = gdk_pixbuf_animation_get_iter(backdrop->priv->animation, NULL);
+
+    /* manually load the first frame */
+    xfce_backdrop_create_final_image(backdrop);
+
     backdrop->priv->image_data = NULL;
     xfce_backdrop_image_data_release(image_data);
     g_free(image_data);
