@@ -96,7 +96,7 @@ struct _XfceDesktopPriv
     XfconfChannel *channel;
     gchar *property_prefix;
     
-    GdkPixmap *bg_pixmap;
+    cairo_surface_t *bg_surface;
     
     gint nworkspaces;
     XfceWorkspace **workspaces;
@@ -114,6 +114,7 @@ struct _XfceDesktopPriv
     gboolean icons_font_size_set;
     guint icons_font_size;
     guint icons_size;
+    gboolean icons_center_text;
     gint  style_refresh_timer;
     GtkWidget *icon_view;
     gdouble system_font_size;
@@ -135,6 +136,7 @@ enum
     PROP_ICON_SIZE,
     PROP_ICON_FONT_SIZE,
     PROP_ICON_FONT_SIZE_SET,
+    PROP_ICON_CENTER_TEXT,
 #endif
     PROP_SINGLE_WORKSPACE_MODE,
     PROP_SINGLE_WORKSPACE_NUMBER,
@@ -159,12 +161,11 @@ static gboolean xfce_desktop_button_release_event(GtkWidget *widget,
                                                   GdkEventButton *evt);
 static gboolean xfce_desktop_popup_menu(GtkWidget *widget);
 
-static gboolean xfce_desktop_expose(GtkWidget *w,
-                                    GdkEventExpose *evt);
+static gboolean xfce_desktop_draw(GtkWidget *w,
+                                  cairo_t *cr);
 static gboolean xfce_desktop_delete_event(GtkWidget *w,
                                           GdkEventAny *evt);
-static void xfce_desktop_style_set(GtkWidget *w,
-                                   GtkStyle *old_style);
+static void xfce_desktop_style_updated(GtkWidget *w);
 
 static void xfce_desktop_set_single_workspace_mode(XfceDesktop *desktop,
                                                    gboolean single_workspace);
@@ -201,7 +202,7 @@ xfce_desktop_ensure_system_font_size(XfceDesktop *desktop)
     desktop->priv->system_font_size = pango_font_description_get_size(pfd);
     /* FIXME: this seems backwards from the documentation */
     if(!pango_font_description_get_size_is_absolute(pfd)) {
-        DBG("dividing by PANGO_SCALE");
+        XF_DEBUG("dividing by PANGO_SCALE");
         desktop->priv->system_font_size /= PANGO_SCALE;
     }
     XF_DEBUG("system font size is %.05f", desktop->priv->system_font_size);
@@ -261,10 +262,15 @@ xfce_desktop_setup_icon_view(XfceDesktop *desktop)
             xfdesktop_icon_view_set_icon_size(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
                                               desktop->priv->icons_size);
         }
+        xfdesktop_icon_view_set_center_text (XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
+                                             desktop->priv->icons_center_text);
+
         gtk_widget_show(desktop->priv->icon_view);
         gtk_container_add(GTK_CONTAINER(desktop), desktop->priv->icon_view);
 
-        g_signal_connect(G_OBJECT(manager), "hidden-state-changed", G_CALLBACK(hidden_state_changed_cb), desktop);
+        if(desktop->priv->icons_style == XFCE_DESKTOP_ICON_STYLE_FILES)
+            g_signal_connect(G_OBJECT(manager), "hidden-state-changed",
+                             G_CALLBACK(hidden_state_changed_cb), desktop);
     }
     
     gtk_widget_queue_draw(GTK_WIDGET(desktop));
@@ -291,19 +297,20 @@ set_imgfile_root_property(XfceDesktop *desktop, const gchar *filename,
                             gdk_atom_intern(property_name, FALSE));
     }
     
-    gdk_error_trap_pop();
+    gdk_error_trap_pop_ignored();
 }
 
 static void
-set_real_root_window_pixmap(GdkScreen *gscreen,
-                            GdkPixmap *pmap)
+set_real_root_window_surface(GdkScreen *gscreen,
+                            cairo_surface_t *surface)
 {
 #ifndef DISABLE_FOR_BUG7442
     Window xid;
     GdkWindow *groot;
+    cairo_pattern_t *pattern;
     
-    xid = GDK_DRAWABLE_XID(pmap);
     groot = gdk_screen_get_root_window(gscreen);
+    xid = GDK_WINDOW_XID(groot);
     
     gdk_error_trap_push();
     
@@ -312,18 +319,21 @@ set_real_root_window_pixmap(GdkScreen *gscreen,
             gdk_atom_intern("_XROOTPMAP_ID", FALSE),
             gdk_atom_intern("PIXMAP", FALSE), 32,
             GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
-    /* and set the root window's BG pixmap, because aterm is somewhat lame. */
-    gdk_window_set_back_pixmap(groot, pmap, FALSE);
+    /* and set the root window's BG surface, because aterm is somewhat lame. */
+    pattern = cairo_pattern_create_for_surface(surface);
+    gdk_window_set_background_pattern(groot, pattern);
+    cairo_pattern_destroy(pattern);
     /* there really should be a standard for this crap... */
 
-    gdk_error_trap_pop();
+    gdk_error_trap_pop_ignored();
 #endif
 }
 
-static GdkPixmap *
-create_bg_pixmap(GdkScreen *gscreen, gpointer user_data)
+static cairo_surface_t *
+create_bg_surface(GdkScreen *gscreen, gpointer user_data)
 {
     XfceDesktop *desktop = user_data;
+    cairo_pattern_t *pattern;
     gint w, h;
 
     TRACE("entering");
@@ -331,9 +341,9 @@ create_bg_pixmap(GdkScreen *gscreen, gpointer user_data)
     g_return_val_if_fail(XFCE_IS_DESKTOP(desktop), NULL);
 
     /* If the workspaces haven't been created yet there's no need to do the
-     * background pixmap */
+     * background surface */
     if(desktop->priv->workspaces == NULL) {
-        DBG("exiting, desktop->priv->workspaces == NULL");
+        XF_DEBUG("exiting, desktop->priv->workspaces == NULL");
         return NULL;
     }
 
@@ -344,28 +354,27 @@ create_bg_pixmap(GdkScreen *gscreen, gpointer user_data)
     gtk_widget_set_size_request(GTK_WIDGET(desktop), w, h);
     gtk_window_resize(GTK_WINDOW(desktop), w, h);
 
-    if(desktop->priv->bg_pixmap)
-        g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
-    desktop->priv->bg_pixmap = gdk_pixmap_new(GDK_DRAWABLE(gtk_widget_get_window(GTK_WIDGET(desktop))),
-                                              w, h, -1);
+    if(desktop->priv->bg_surface)
+        cairo_surface_destroy(desktop->priv->bg_surface);
+    desktop->priv->bg_surface = gdk_window_create_similar_surface(
+                                    gtk_widget_get_window(GTK_WIDGET(desktop)),
+                                                          CAIRO_CONTENT_COLOR_ALPHA, w, h);
 
-    if(!GDK_IS_PIXMAP(desktop->priv->bg_pixmap))
-        return NULL;
+    pattern = cairo_pattern_create_for_surface(desktop->priv->bg_surface);
+    gdk_window_set_background_pattern(gtk_widget_get_window(GTK_WIDGET(desktop)), pattern);
+    cairo_pattern_destroy(pattern);
 
-    gdk_window_set_back_pixmap(gtk_widget_get_window(GTK_WIDGET(desktop)),
-                               desktop->priv->bg_pixmap, FALSE);
-
-    return desktop->priv->bg_pixmap;
+    return desktop->priv->bg_surface;
 }
 
 static void
 backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(user_data);
-    GdkPixmap *pmap = desktop->priv->bg_pixmap;
+    cairo_surface_t *surface = desktop->priv->bg_surface;
     GdkScreen *gscreen = desktop->priv->gscreen;
     GdkRectangle rect;
-    GdkRegion *clip_region = NULL;
+    cairo_region_t *clip_region = NULL;
     gint i, monitor = -1, current_workspace;
 #ifdef G_ENABLE_DEBUG
     gchar *monitor_name = NULL;
@@ -435,7 +444,7 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 
     if(monitor > 0
        && !xfce_workspace_get_xinerama_stretch(desktop->priv->workspaces[current_workspace])) {
-        clip_region = gdk_region_rectangle(&rect);
+        clip_region = cairo_region_create_rectangle(&rect);
 
         XF_DEBUG("clip_region: x: %d, y: %d, w: %d, h: %d",
                  rect.x, rect.y, rect.width, rect.height);
@@ -446,30 +455,30 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
          */
         for(i = 0; i < monitor; i++) {
             GdkRectangle previous_monitor;
-            GdkRegion *previous_region;
+            cairo_region_t *previous_region;
             gdk_screen_get_monitor_geometry(gscreen, i, &previous_monitor);
 
             XF_DEBUG("previous_monitor: x: %d, y: %d, w: %d, h: %d",
                      previous_monitor.x, previous_monitor.y,
                      previous_monitor.width, previous_monitor.height);
 
-            previous_region = gdk_region_rectangle(&previous_monitor);
+            previous_region = cairo_region_create_rectangle(&previous_monitor);
 
-            gdk_region_subtract(clip_region, previous_region);
+            cairo_region_subtract(clip_region, previous_region);
 
-            gdk_region_destroy(previous_region);
+            cairo_region_destroy(previous_region);
         }
     }
 
     if(clip_region != NULL) {
         /* Update the area to redraw to limit the icons/area painted */
-        gdk_region_get_clipbox(clip_region, &rect);
+        cairo_region_get_extents(clip_region, &rect);
         XF_DEBUG("area to update: x: %d, y: %d, w: %d, h: %d",
                  rect.x, rect.y, rect.width, rect.height);
     }
 
     if(rect.width != 0 && rect.height != 0) {
-        /* get the composited backdrop pixmap */
+        /* get the composited backdrop pixbuf */
         GdkPixbuf *pix = xfce_backdrop_get_pixbuf(backdrop);
         cairo_t *cr;
 
@@ -478,26 +487,26 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
             xfce_backdrop_generate_async(backdrop);
 
             if(clip_region != NULL)
-                gdk_region_destroy(clip_region);
+                cairo_region_destroy(clip_region);
 
             return;
         }
 
-        /* Create the background pixmap if it isn't already */
-        if(!GDK_IS_PIXMAP(pmap)) {
-            pmap = create_bg_pixmap(gscreen, desktop);
+        /* Create the background surface if it isn't already */
+        if(!desktop->priv->bg_surface) {
+            surface = create_bg_surface(gscreen, desktop);
 
-            if(!GDK_IS_PIXMAP(pmap)) {
+            if(!surface) {
                 g_object_unref(pix);
 
                 if(clip_region != NULL)
-                    gdk_region_destroy(clip_region);
+                    cairo_region_destroy(clip_region);
 
                 return;
             }
         }
 
-        cr = gdk_cairo_create(GDK_DRAWABLE(pmap));
+        cr = cairo_create(surface);
         gdk_cairo_set_source_pixbuf(cr, pix, rect.x, rect.y);
 
         /* clip the area so we don't draw over a previous wallpaper */
@@ -517,7 +526,7 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
                                   monitor);
 
         /* do this again so apps watching the root win notice the update */
-        set_real_root_window_pixmap(gscreen, pmap);
+        set_real_root_window_surface(gscreen, surface);
 
         g_object_unref(G_OBJECT(pix));
         cairo_destroy(cr);
@@ -525,7 +534,7 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     }
 
     if(clip_region != NULL)
-        gdk_region_destroy(clip_region);
+        cairo_region_destroy(clip_region);
 }
 
 static void
@@ -544,10 +553,10 @@ screen_size_changed_cb(GdkScreen *gscreen, gpointer user_data)
     if(current_workspace < 0)
         return;
 
-    /* release the bg_pixmap since the dimensions may have changed */
-    if(desktop->priv->bg_pixmap) {
-        g_object_unref(desktop->priv->bg_pixmap);
-        desktop->priv->bg_pixmap = NULL;
+    /* release the bg_surface since the dimensions may have changed */
+    if(desktop->priv->bg_surface) {
+        cairo_surface_destroy(desktop->priv->bg_surface);
+        desktop->priv->bg_surface = NULL;
     }
 
     /* special case for 1 backdrop to handle xinerama stretching */
@@ -598,13 +607,30 @@ workspace_backdrop_changed_cb(XfceWorkspace *workspace,
                               gpointer user_data)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(user_data);
+    gint current_workspace = 0, monitor = 0, i;
 
     TRACE("entering");
 
     g_return_if_fail(XFCE_IS_WORKSPACE(workspace) && XFCE_IS_BACKDROP(backdrop));
 
-    if(xfce_desktop_get_current_workspace(desktop) == xfce_workspace_get_workspace_num(workspace))
-        backdrop_changed_cb(backdrop, user_data);
+    current_workspace = xfce_desktop_get_current_workspace(desktop);
+
+    /* Find out which monitor the backdrop is on */
+    for(i = 0; i < xfce_desktop_get_n_monitors(desktop); i++) {
+        if(backdrop == xfce_workspace_get_backdrop(desktop->priv->workspaces[current_workspace], i)) {
+            monitor = i;
+            break;
+        }
+    }
+
+    if(xfce_desktop_get_current_workspace(desktop) == xfce_workspace_get_workspace_num(workspace)) {
+        /* Update the backdrop!
+         * In spanning mode, ignore updates to monitors other than the primary
+         */
+        if(!xfce_workspace_get_xinerama_stretch(workspace) || monitor == 0) {
+            backdrop_changed_cb(backdrop, user_data);
+        }
+    }
 }
 
 static void
@@ -777,10 +803,10 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
     widget_class->unrealize = xfce_desktop_unrealize;
     widget_class->button_press_event = xfce_desktop_button_press_event;
     widget_class->button_release_event = xfce_desktop_button_release_event;
-    widget_class->expose_event = xfce_desktop_expose;
+    widget_class->draw = xfce_desktop_draw;
     widget_class->delete_event = xfce_desktop_delete_event;
     widget_class->popup_menu = xfce_desktop_popup_menu;
-    widget_class->style_set = xfce_desktop_style_set;
+    widget_class->style_updated = xfce_desktop_style_updated;
     
     signals[SIG_POPULATE_ROOT_MENU] = g_signal_new("populate-root-menu",
                                                    XFCE_TYPE_DESKTOP,
@@ -842,6 +868,13 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
                                                          FALSE,
                                                          XFDESKTOP_PARAM_FLAGS));
 
+    g_object_class_install_property(gobject_class, PROP_ICON_CENTER_TEXT,
+                                    g_param_spec_boolean("icon-center-text",
+                                                         "icon center text",
+                                                         "icon center text",
+                                                         TRUE,
+                                                         XFDESKTOP_PARAM_FLAGS));
+
 #endif /* ENABLE_DESKTOP_ICONS */
 
     g_object_class_install_property(gobject_class, PROP_SINGLE_WORKSPACE_MODE,
@@ -922,6 +955,11 @@ xfce_desktop_set_property(GObject *object,
                                                 g_value_get_boolean(value));
             break;
 
+        case PROP_ICON_CENTER_TEXT:
+            xfce_desktop_set_center_text(desktop,
+                                         g_value_get_boolean(value));
+            break;
+
 #endif
         case PROP_SINGLE_WORKSPACE_MODE:
             xfce_desktop_set_single_workspace_mode(desktop,
@@ -965,6 +1003,10 @@ xfce_desktop_get_property(GObject *object,
             g_value_set_boolean(value, desktop->priv->icons_font_size_set);
             break;
 
+        case PROP_ICON_CENTER_TEXT:
+            g_value_set_boolean(value, desktop->priv->icons_center_text);
+            break;
+
 #endif
         case PROP_SINGLE_WORKSPACE_MODE:
             g_value_set_boolean(value, desktop->priv->single_workspace_mode);
@@ -1103,7 +1145,7 @@ xfce_desktop_unrealize(GtkWidget *widget)
 #ifndef DISABLE_FOR_BUG7442
     gdk_property_delete(groot, gdk_atom_intern("_XROOTPMAP_ID", FALSE));
     gdk_property_delete(groot, gdk_atom_intern("ESETROOT_PMAP_ID", FALSE));
-    gdk_window_set_back_pixmap(groot, NULL, FALSE);
+    gdk_window_set_background_pattern(groot, NULL);
 #endif
 
     if(desktop->priv->workspaces) {
@@ -1117,16 +1159,15 @@ xfce_desktop_unrealize(GtkWidget *widget)
     }
 
     gdk_flush();
-    gdk_error_trap_pop();
+    gdk_error_trap_pop_ignored();
 
-    if(desktop->priv->bg_pixmap) {
-        g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
-        desktop->priv->bg_pixmap = NULL;
+    if(desktop->priv->bg_surface) {
+        cairo_surface_destroy(desktop->priv->bg_surface);
+        desktop->priv->bg_surface = NULL;
     }
     
     gtk_window_set_icon(GTK_WINDOW(widget), NULL);
 
-    gtk_style_detach(gtk_widget_get_style(widget));
     g_object_unref(G_OBJECT(gtk_widget_get_window(widget)));
     gtk_widget_set_window(widget, NULL);
 
@@ -1143,7 +1184,7 @@ xfce_desktop_button_press_event(GtkWidget *w,
     guint state = evt->state;
     XfceDesktop *desktop = XFCE_DESKTOP(w);
 
-    TRACE("entering");
+    DBG("entering");
 
     g_return_val_if_fail(XFCE_IS_DESKTOP(w), FALSE);
 
@@ -1155,11 +1196,12 @@ xfce_desktop_button_press_event(GtkWidget *w,
                 return FALSE;
 #endif
             /* no icons on the desktop, grab the focus and pop up the menu */
-            if(!gtk_widget_has_grab(w))
+            if(!gtk_widget_has_grab(w)) {
                 gtk_grab_add(w);
 
                 xfce_desktop_popup_root_menu(desktop, button, evt->time);
                 return TRUE;
+            }
         } else if(button == 2 || (button == 1 && (state & GDK_SHIFT_MASK)
                                   && (state & GDK_CONTROL_MASK)))
         {
@@ -1179,7 +1221,7 @@ static gboolean
 xfce_desktop_button_release_event(GtkWidget *w,
                                   GdkEventButton *evt)
 {
-    TRACE("entering");
+    DBG("entering");
 
     gtk_grab_remove(w);
 
@@ -1194,7 +1236,7 @@ xfce_desktop_popup_menu(GtkWidget *w)
     GdkEventButton *evt;
     guint button, etime;
 
-    TRACE("entering");
+    DBG("entering");
 
     evt = (GdkEventButton *)gtk_get_current_event();
     if(evt && GDK_BUTTON_PRESS == evt->type) {
@@ -1212,24 +1254,18 @@ xfce_desktop_popup_menu(GtkWidget *w)
 }
 
 static gboolean
-xfce_desktop_expose(GtkWidget *w,
-                    GdkEventExpose *evt)
+xfce_desktop_draw(GtkWidget *w,
+                  cairo_t *cr)
 {
     GList *children, *l;
     
     /*TRACE("entering");*/
     
-    if(evt->count != 0)
-        return FALSE;
-    
-    gdk_window_clear_area(gtk_widget_get_window(w), evt->area.x, evt->area.y,
-                          evt->area.width, evt->area.height);
-    
     children = gtk_container_get_children(GTK_CONTAINER(w));
     for(l = children; l; l = l->next) {
-        gtk_container_propagate_expose(GTK_CONTAINER(w),
-                                       GTK_WIDGET(l->data),
-                                       evt);
+        gtk_container_propagate_draw(GTK_CONTAINER(w),
+                                     GTK_WIDGET(l->data),
+                                     cr);
     }
     g_list_free(children);
     
@@ -1251,6 +1287,7 @@ static gboolean
 style_refresh_cb(gpointer *w)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(w);
+    cairo_pattern_t *pattern;
     gdouble old_font_size;
 
     TRACE("entering");
@@ -1265,8 +1302,12 @@ style_refresh_cb(gpointer *w)
     if(desktop->priv->workspaces == NULL)
         return FALSE;
 
-    if(GDK_IS_WINDOW(desktop->priv->bg_pixmap))
-        gdk_window_set_back_pixmap(gtk_widget_get_window(GTK_WIDGET(desktop)), desktop->priv->bg_pixmap, FALSE);
+    if(desktop->priv->bg_surface) {
+        pattern = cairo_pattern_create_for_surface(desktop->priv->bg_surface);
+        gdk_window_set_background_pattern(gtk_widget_get_window(GTK_WIDGET(desktop)),
+                                          pattern);
+        cairo_pattern_destroy(pattern);
+    }
 
     gtk_widget_queue_draw(GTK_WIDGET(desktop));
 
@@ -1274,8 +1315,7 @@ style_refresh_cb(gpointer *w)
         return FALSE;
 
     /* reset the icon view style */
-    gtk_widget_set_style(desktop->priv->icon_view, NULL);
-    gtk_widget_reset_rc_styles(desktop->priv->icon_view);
+    gtk_widget_reset_style(desktop->priv->icon_view);
 
     old_font_size = desktop->priv->system_font_size;
     if(xfce_desktop_ensure_system_font_size(desktop) != old_font_size
@@ -1290,7 +1330,7 @@ style_refresh_cb(gpointer *w)
 #endif
 
 static void
-xfce_desktop_style_set(GtkWidget *w, GtkStyle *old_style)
+xfce_desktop_style_updated(GtkWidget *w)
 {
 #ifdef ENABLE_DESKTOP_ICONS
     XfceDesktop *desktop = XFCE_DESKTOP(w);
@@ -1326,6 +1366,9 @@ xfce_desktop_connect_settings(XfceDesktop *desktop)
     xfconf_g_property_bind(channel, ICONS_PREFIX "use-custom-font-size",
                            G_TYPE_BOOLEAN,
                            G_OBJECT(desktop), "icon-font-size-set");
+    xfconf_g_property_bind(channel, ICONS_PREFIX "center-text",
+                           G_TYPE_BOOLEAN,
+                           G_OBJECT(desktop), "icon-center-text");
 
     xfce_desktop_thaw_updates(desktop);
 #undef ICONS_PREFIX
@@ -1570,6 +1613,23 @@ xfce_desktop_set_use_icon_font_size(XfceDesktop *desktop,
 #endif
 }
 
+void
+xfce_desktop_set_center_text (XfceDesktop *desktop,
+                              gboolean center_text)
+{
+    g_return_if_fail(XFCE_IS_DESKTOP(desktop));
+    
+#ifdef ENABLE_DESKTOP_ICONS
+    if(center_text == desktop->priv->icons_center_text)
+        return;
+
+    desktop->priv->icons_center_text = center_text;
+    if(desktop->priv->icon_view) {
+        xfdesktop_icon_view_set_center_text (XFDESKTOP_ICON_VIEW(desktop->priv->icon_view), center_text);
+    }
+#endif
+}
+
 static void
 xfce_desktop_set_single_workspace_mode(XfceDesktop *desktop,
                                        gboolean single_workspace)
@@ -1581,7 +1641,7 @@ xfce_desktop_set_single_workspace_mode(XfceDesktop *desktop,
 
     desktop->priv->single_workspace_mode = single_workspace;
 
-    DBG("single_workspace_mode now %s", single_workspace ? "TRUE" : "FALSE");
+    XF_DEBUG("single_workspace_mode now %s", single_workspace ? "TRUE" : "FALSE");
 
 
     /* If the desktop has been realized then fake a screen size change to
@@ -1599,7 +1659,7 @@ xfce_desktop_set_single_workspace_number(XfceDesktop *desktop,
     if(workspace_num == desktop->priv->single_workspace_num)
         return;
 
-    DBG("single_workspace_num now %d", workspace_num);
+    XF_DEBUG("single_workspace_num now %d", workspace_num);
 
     desktop->priv->single_workspace_num = workspace_num;
 
@@ -1651,7 +1711,7 @@ xfce_desktop_do_menu_popup(XfceDesktop *desktop,
     GtkWidget *menu;
     GList *menu_children;
     
-    TRACE("entering");
+    DBG("entering");
     
     if(gtk_widget_has_screen(GTK_WIDGET(desktop)))
         screen = gtk_widget_get_screen(GTK_WIDGET(desktop));
@@ -1660,6 +1720,7 @@ xfce_desktop_do_menu_popup(XfceDesktop *desktop,
 
     menu = gtk_menu_new();
     gtk_menu_set_screen(GTK_MENU(menu), screen);
+    gtk_menu_set_reserve_toggle_size (GTK_MENU (menu), FALSE);
     g_signal_connect_swapped(G_OBJECT(menu), "deactivate",
                              G_CALLBACK(g_idle_add),
                              (gpointer)xfce_desktop_menu_destroy_idled);
@@ -1682,20 +1743,16 @@ xfce_desktop_do_menu_popup(XfceDesktop *desktop,
     if(activate_time == 0)
         activate_time = gtk_get_current_event_time();
 
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
+    xfce_gtk_menu_popup_until_mapped(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
 }
 
+
 void
 xfce_desktop_popup_root_menu(XfceDesktop *desktop,
                              guint button,
                              guint activate_time)
 {
-    TRACE("entering");
-
-    /* If it's launched by xfdesktop --menu we won't have an event time.
-     * Grab the keyboard focus */
-    if(activate_time == 0)
-        activate_time = xfdesktop_popup_keyboard_grab_available(gtk_widget_get_window(GTK_WIDGET(desktop)));
+    DBG("entering");
 
     xfce_desktop_do_menu_popup(desktop, button, activate_time,
                                signals[SIG_POPULATE_ROOT_MENU]);
@@ -1707,10 +1764,7 @@ xfce_desktop_popup_secondary_root_menu(XfceDesktop *desktop,
                                        guint button,
                                        guint activate_time)
 {
-    /* If it's launched by xfdesktop --windowlist we won't have an event time.
-     * Grab the keyboard focus */
-    if(activate_time == 0)
-        activate_time = xfdesktop_popup_keyboard_grab_available(gtk_widget_get_window(GTK_WIDGET(desktop)));
+    DBG("entering");
 
     xfce_desktop_do_menu_popup(desktop, button, activate_time,
                                signals[SIG_POPULATE_SECONDARY_ROOT_MENU]);
@@ -1744,6 +1798,8 @@ xfce_desktop_refresh(XfceDesktop *desktop, gboolean advance_wallpaper)
             /* We need to trigger a new wallpaper event */
             xfce_backdrop_force_cycle(backdrop);
         } else {
+            /* Reinitialize wallpaper */
+            xfce_backdrop_clear_cached_image(backdrop);
             /* Fake a changed event so we redraw the wallpaper */
             backdrop_changed_cb(backdrop, desktop);
         }
