@@ -84,7 +84,6 @@ struct _XfceWorkspacePriv
     guint nbackdrops;
     gboolean xinerama_stretch;
     XfceBackdrop **backdrops;
-    gboolean cache_pixbufs;
 
     gulong *first_color_id;
     gulong *second_color_id;
@@ -145,12 +144,19 @@ xfce_workspace_set_xfconf_property_string(XfceWorkspace *workspace,
 {
     XfconfChannel *channel = workspace->priv->channel;
     char buf[1024];
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    GdkDisplay *display;
+#endif
     gchar *monitor_name = NULL;
 
     TRACE("entering");
 
-    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen,
-                                                    monitor_num);
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    display = gdk_display_manager_get_default_display(gdk_display_manager_get());
+    monitor_name = g_strdup(gdk_monitor_get_model(gdk_display_get_monitor(display, monitor_num)));
+#else
+    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen, monitor_num);
+#endif
 
     /* Get the backdrop's image property */
     if(monitor_name == NULL) {
@@ -158,12 +164,12 @@ xfce_workspace_set_xfconf_property_string(XfceWorkspace *workspace,
                    workspace->priv->property_prefix, monitor_num, workspace->priv->workspace_num, property);
     } else {
         g_snprintf(buf, sizeof(buf), "%smonitor%s/workspace%d/%s",
-                   workspace->priv->property_prefix, monitor_name, workspace->priv->workspace_num, property);
+                   workspace->priv->property_prefix, xfdesktop_remove_whitspaces(monitor_name), workspace->priv->workspace_num, property);
 
         g_free(monitor_name);
     }
 
-    DBG("setting %s to %s", buf, value);
+    XF_DEBUG("setting %s to %s", buf, value);
 
     xfconf_channel_set_string(channel, buf, value);
 }
@@ -177,14 +183,21 @@ xfce_workspace_set_xfconf_property_value(XfceWorkspace *workspace,
     XfconfChannel *channel = workspace->priv->channel;
     char buf[1024];
     gchar *monitor_name = NULL;
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    GdkDisplay *display;
+#endif
 #ifdef G_ENABLE_DEBUG
     gchar *contents = NULL;
 #endif
 
     TRACE("entering");
 
-    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen,
-                                                    monitor_num);
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    display = gdk_display_manager_get_default_display(gdk_display_manager_get());
+    monitor_name = g_strdup(gdk_monitor_get_model(gdk_display_get_monitor(display, monitor_num)));
+#else
+    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen, monitor_num);
+#endif
 
     /* Get the backdrop's image property */
     if(monitor_name == NULL) {
@@ -192,7 +205,7 @@ xfce_workspace_set_xfconf_property_value(XfceWorkspace *workspace,
                    workspace->priv->property_prefix, monitor_num, workspace->priv->workspace_num, property);
     } else {
         g_snprintf(buf, sizeof(buf), "%smonitor%s/workspace%d/%s",
-                   workspace->priv->property_prefix, monitor_name, workspace->priv->workspace_num, property);
+                   workspace->priv->property_prefix, xfdesktop_remove_whitspaces(monitor_name), workspace->priv->workspace_num, property);
 
         g_free(monitor_name);
     }
@@ -221,7 +234,7 @@ xfce_workspace_change_backdrop(XfceWorkspace *workspace,
     for(i = 0; i < workspace->priv->nbackdrops; ++i) {
         if(backdrop == workspace->priv->backdrops[i]) {
             monitor_num = i;
-            DBG("monitor_num %d", monitor_num);
+            XF_DEBUG("monitor_num %d", monitor_num);
             break;
         }
     }
@@ -306,7 +319,11 @@ xfce_workspace_monitors_changed(XfceWorkspace *workspace,
         /* When spanning screens we only need one backdrop */
         n_monitors = 1;
     } else {
-        n_monitors = gdk_screen_get_n_monitors(gscreen);
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+        n_monitors = gdk_display_get_n_monitors(gdk_screen_get_display(workspace->priv->gscreen));
+#else
+        n_monitors = gdk_screen_get_n_monitors(workspace->priv->gscreen);
+#endif
     }
 
     /* Remove all backdrops so that the correct monitor is added/removed and
@@ -325,7 +342,7 @@ xfce_workspace_monitors_changed(XfceWorkspace *workspace,
     workspace->priv->nbackdrops = n_monitors;
 
     for(i = 0; i < n_monitors; ++i) {
-        DBG("Adding workspace %d backdrop %d", workspace->priv->workspace_num, i);
+        XF_DEBUG("Adding workspace %d backdrop %d", workspace->priv->workspace_num, i);
 
         workspace->priv->backdrops[i] = xfce_backdrop_new(vis);
         xfce_workspace_connect_backdrop_settings(workspace,
@@ -456,6 +473,12 @@ xfce_workspace_migrate_backdrop_first_color(XfceWorkspace *workspace,
 
     TRACE("entering");
 
+    /* TODO: migrate from color1 (and older) to the new rgba1 */
+    if(TRUE) {
+        TRACE("warning: we aren't migrating from GdkColor to GdkRGBA yet");
+        return;
+    }
+
     /* Use the old property format */
     g_snprintf(buf, sizeof(buf), "%smonitor%d/",
                workspace->priv->property_prefix, monitor);
@@ -482,6 +505,12 @@ xfce_workspace_migrate_backdrop_second_color(XfceWorkspace *workspace,
 
     TRACE("entering");
 
+    /* TODO: migrate from color1 (and older) to the new rgba1 */
+    if(TRUE) {
+        TRACE("warning: we aren't migrating from GdkColor to GdkRGBA yet");
+        return;
+    }
+
     /* Use the old property format */
     g_snprintf(buf, sizeof(buf), "%smonitor%d/",
                workspace->priv->property_prefix, monitor);
@@ -516,7 +545,7 @@ xfce_workspace_migrate_backdrop_image(XfceWorkspace *workspace,
     /* Try to lookup the old backdrop */
     xfconf_channel_get_property(channel, buf, &value);
 
-    DBG("looking at %s", buf);
+    XF_DEBUG("looking at %s", buf);
 
     /* Either there was a backdrop to migrate from or we use the backdrop
      * we provide as a default */
@@ -583,9 +612,9 @@ xfce_workspace_migrate_backdrop_image_style(XfceWorkspace *workspace,
         xfce_workspace_set_xfconf_property_value(workspace, monitor, "image-style", &value);
         g_value_unset(&value);
     } else {
-        /* If no value was ever set default to stretched */
+        /* If no value was ever set default to zoomed */
         g_value_init(&value, G_TYPE_INT);
-        g_value_set_int(&value, XFCE_BACKDROP_IMAGE_STRETCHED);
+        g_value_set_int(&value, XFCE_BACKDROP_IMAGE_ZOOMED);
         xfce_workspace_set_xfconf_property_value(workspace, monitor, "image-style", &value);
         g_value_unset(&value);
     }
@@ -599,22 +628,31 @@ xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
     XfconfChannel *channel = workspace->priv->channel;
     char buf[1024];
     gint pp_len;
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    GdkDisplay *display;
+#endif
     gchar *monitor_name = NULL;
 
     TRACE("entering");
 
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    display = gdk_display_manager_get_default_display(gdk_display_manager_get());
+    monitor_name = g_strdup(gdk_monitor_get_model(gdk_display_get_monitor(display, monitor)));
+#else
     monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen, monitor);
+#endif
 
     if(monitor_name == NULL) {
         g_snprintf(buf, sizeof(buf), "%smonitor%d/workspace%d/",
                    workspace->priv->property_prefix, monitor, workspace->priv->workspace_num);
     } else {
         g_snprintf(buf, sizeof(buf), "%smonitor%s/workspace%d/",
-                   workspace->priv->property_prefix, monitor_name, workspace->priv->workspace_num);
-    }
+                   workspace->priv->property_prefix, xfdesktop_remove_whitspaces(monitor_name), workspace->priv->workspace_num);
+        g_free(monitor_name);
+   }
     pp_len = strlen(buf);
 
-    DBG("prefix string: %s", buf);
+    XF_DEBUG("prefix string: %s", buf);
 
     g_strlcat(buf, "color-style", sizeof(buf));
     if(!xfconf_channel_has_property(channel, buf)) {
@@ -624,19 +662,19 @@ xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
                            G_OBJECT(backdrop), "color-style");
 
     buf[pp_len] = 0;
-    g_strlcat(buf, "color1", sizeof(buf));
+    g_strlcat(buf, "rgba1", sizeof(buf));
     if(!xfconf_channel_has_property(channel, buf)) {
         xfce_workspace_migrate_backdrop_first_color(workspace, backdrop, monitor);
     }
-    workspace->priv->first_color_id[monitor] = xfconf_g_property_bind_gdkcolor(channel, buf,
+    workspace->priv->first_color_id[monitor] = xfconf_g_property_bind_gdkrgba(channel, buf,
                                                             G_OBJECT(backdrop), "first-color");
 
     buf[pp_len] = 0;
-    g_strlcat(buf, "color2", sizeof(buf));
+    g_strlcat(buf, "rgba2", sizeof(buf));
     if(!xfconf_channel_has_property(channel, buf)) {
         xfce_workspace_migrate_backdrop_second_color(workspace, backdrop, monitor);
     }
-    workspace->priv->second_color_id[monitor] = xfconf_g_property_bind_gdkcolor(channel, buf,
+    workspace->priv->second_color_id[monitor] = xfconf_g_property_bind_gdkrgba(channel, buf,
                                                             G_OBJECT(backdrop), "second-color");
 
     buf[pp_len] = 0;
@@ -667,6 +705,11 @@ xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
     xfconf_g_property_bind(channel, buf, G_TYPE_BOOLEAN,
                            G_OBJECT(backdrop), "backdrop-cycle-random-order");
 
+    buf[pp_len] = 0;
+    g_strlcat(buf, "backdrop-do-animations", sizeof(buf));
+    xfconf_g_property_bind(channel, buf, G_TYPE_BOOLEAN,
+                           G_OBJECT(backdrop), "backdrop-do-animations");
+
     buf[pp_len] = 0;
     g_strlcat(buf, "last-image", sizeof(buf));
     if(!xfconf_channel_has_property(channel, buf)) {
@@ -675,11 +718,6 @@ xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
     xfconf_g_property_bind(channel, buf, G_TYPE_STRING,
                            G_OBJECT(backdrop), "image-filename");
 
-    /* determine if the backdrop will be required to keep a ref of it's pixbuf
-     * when it generates one */
-    xfce_backdrop_set_cache_pixbuf(backdrop, workspace->priv->cache_pixbufs);
-
-    g_free(monitor_name);
 }
 
 static void
@@ -702,7 +740,11 @@ xfce_workspace_remove_backdrops(XfceWorkspace *workspace)
 
     g_return_if_fail(XFCE_IS_WORKSPACE(workspace));
 
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    n_monitors = gdk_display_get_n_monitors(gdk_screen_get_display(workspace->priv->gscreen));
+#else
     n_monitors = gdk_screen_get_n_monitors(workspace->priv->gscreen);
+#endif
 
     for(i = 0; i < n_monitors && i < workspace->priv->nbackdrops; ++i) {
         xfce_workspace_disconnect_backdrop_settings(workspace,
@@ -775,49 +817,6 @@ xfce_workspace_set_workspace_num(XfceWorkspace *workspace, gint number)
     workspace->priv->workspace_num = number;
 }
 
-/**
- * xfce_workspace_set_cache_pixbufs:
- * @workspace: An #XfceWorkspace.
- * @cache_pixbuf: When TRUE XfceWorkspace will have all it's backdrops keep
- *                a reference to the current pixbuf.
- *
- * This function will control whether XfceWorkspace's backdrops keep a reference
- * to their respective pixbufs or not. Setting it to TRUE is useful for the
- * per-workspace-wallpapers, FALSE will save memory for single workspace mode.
- **/
-void
-xfce_workspace_set_cache_pixbufs(XfceWorkspace *workspace,
-                                 gboolean cache_pixbuf)
-{
-    guint i;
-    guint n_monitors;
-
-    g_return_if_fail(XFCE_IS_WORKSPACE(workspace));
-
-    /* If nothing changed then avoid doing any work */
-    if(workspace->priv->cache_pixbufs == cache_pixbuf)
-        return;
-
-    workspace->priv->cache_pixbufs = cache_pixbuf;
-
-    DBG("cache_pixbuf now %s", cache_pixbuf ? "TRUE" : "FALSE");
-
-    n_monitors = gdk_screen_get_n_monitors(workspace->priv->gscreen);
-
-    /* update all the backdrops */
-    for(i = 0; i < n_monitors && i < workspace->priv->nbackdrops; ++i) {
-        xfce_backdrop_set_cache_pixbuf(workspace->priv->backdrops[i], cache_pixbuf);
-    }
-}
-
-gboolean
-xfce_workspace_get_cache_pixbufs(XfceWorkspace *workspace)
-{
-    g_return_val_if_fail(XFCE_IS_WORKSPACE(workspace), FALSE);
-
-    return workspace->priv->cache_pixbufs;
-}
-
 /**
  * xfce_workspace_get_backdrop:
  * @workspace: An #XfceWorkspace.
